@@ -474,12 +474,10 @@ g_buffered_output_stream_flush (GOutputStream  *stream,
                                 GError        **error)
 {
   GBufferedOutputStream *bstream;
-  GBufferedOutputStreamPrivate *priv;
   GOutputStream                *base_stream;
   gboolean res;
 
   bstream = G_BUFFERED_OUTPUT_STREAM (stream);
-  priv = bstream->priv;
   base_stream = G_FILTER_OUTPUT_STREAM (stream)->base_stream;
 
   res = flush_buffer (bstream, cancellable, error);
@@ -498,21 +496,21 @@ g_buffered_output_stream_close (GOutputStream  *stream,
                                 GError        **error)
 {
   GBufferedOutputStream        *bstream;
-  GBufferedOutputStreamPrivate *priv;
   GOutputStream                *base_stream;
   gboolean                      res;
 
   bstream = G_BUFFERED_OUTPUT_STREAM (stream);
-  priv = bstream->priv;
   base_stream = G_FILTER_OUTPUT_STREAM (bstream)->base_stream;
-
   res = flush_buffer (bstream, cancellable, error);
 
-  /* report the first error but still close the stream */
-  if (res)
-    res = g_output_stream_close (base_stream, cancellable, error); 
-  else
-    g_output_stream_close (base_stream, cancellable, NULL); 
+  if (g_filter_output_stream_get_close_base_stream (G_FILTER_OUTPUT_STREAM (stream)))
+    {
+      /* report the first error but still close the stream */
+      if (res)
+        res = g_output_stream_close (base_stream, cancellable, error);
+      else
+        g_output_stream_close (base_stream, cancellable, NULL);
+    }
 
   return res;
 }
@@ -569,10 +567,13 @@ flush_buffer_thread (GSimpleAsyncResult *result,
       /* if flushing the buffer or the stream returned 
        * an error report that first error but still try 
        * close the stream */
-      if (res == FALSE)
-        g_output_stream_close (base_stream, cancellable, NULL);
-      else 
-        res = g_output_stream_close (base_stream, cancellable, &error);
+      if (g_filter_output_stream_get_close_base_stream (G_FILTER_OUTPUT_STREAM (stream)))
+        {
+          if (res == FALSE)
+            g_output_stream_close (base_stream, cancellable, NULL);
+          else 
+            res = g_output_stream_close (base_stream, cancellable, &error);
+        }
     }
 
   if (res == FALSE)
@@ -758,7 +759,7 @@ g_buffered_output_stream_close_finish (GOutputStream        *stream,
   simple = G_SIMPLE_ASYNC_RESULT (result);
 
   g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == 
-            g_buffered_output_stream_flush_async);
+            g_buffered_output_stream_close_async);
 
   return TRUE;
 }
