@@ -20,11 +20,12 @@
  * Author: Christian Kellner <gicmo@gnome.org>
  */
 
-#include <config.h>
+#include "config.h"
 #include "gmemoryoutputstream.h"
 #include "goutputstream.h"
 #include "gseekable.h"
 #include "gsimpleasyncresult.h"
+#include "gioerror.h"
 #include "string.h"
 #include "glibintl.h"
 
@@ -47,6 +48,7 @@ struct _GMemoryOutputStreamPrivate {
   
   gpointer       data;
   gsize          len;
+  gsize          valid_len; /* The part of data that has been written to */
 
   goffset        pos;
 
@@ -136,9 +138,8 @@ g_memory_output_stream_finalize (GObject *object)
   
   if (priv->destroy)
     priv->destroy (priv->data);
-    
-  if (G_OBJECT_CLASS (g_memory_output_stream_parent_class)->finalize)
-    (*G_OBJECT_CLASS (g_memory_output_stream_parent_class)->finalize) (object);
+
+  G_OBJECT_CLASS (g_memory_output_stream_parent_class)->finalize (object);
 }
 
 static void
@@ -205,6 +206,7 @@ g_memory_output_stream_new (gpointer       data,
   priv->destroy = destroy;
 
   priv->pos = 0;
+  priv->valid_len = 0;
 
   return stream;
 }
@@ -235,8 +237,8 @@ g_memory_output_stream_get_data (GMemoryOutputStream *ostream)
  * Gets the size of the currently allocated data area (availible from
  * g_memory_output_stream_get_data()). If the stream isn't
  * growable (no realloc was passed to g_memory_output_stream_new()) then
- * this is the max size of the stream and further writes
- * will return G_IO_ERROR_NO_SPACE.
+ * this is the maximum size of the stream and further writes
+ * will return %G_IO_ERROR_NO_SPACE.
  *
  * Note that for growable streams the returned size may become invalid on
  * the next write or truncate operation on the stream.
@@ -271,7 +273,7 @@ g_memory_output_stream_get_data_size (GMemoryOutputStream *ostream)
 {
   g_return_val_if_fail (G_IS_MEMORY_OUTPUT_STREAM (ostream), 0);
   
-  return ostream->priv->pos;
+  return ostream->priv->valid_len;
 }
 
 
@@ -282,10 +284,10 @@ array_check_boundary (GMemoryOutputStream  *stream,
 {
   if (size > G_MAXUINT)
     {
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_FAILED,
-                   _("Reached maximum data array limit"));
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_FAILED,
+                           _("Reached maximum data array limit"));
 
       return FALSE;
     }
@@ -317,26 +319,26 @@ array_resize (GMemoryOutputStream  *ostream,
 	  priv->pos < priv->len)
 	return TRUE; /* Short write */
       
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_NO_SPACE,
-                   _("Memory output stream not resizable"));
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_NO_SPACE,
+                           _("Memory output stream not resizable"));
       return FALSE;
     }
 
   len = priv->len;
   data = priv->realloc_fn (priv->data, size);
 
-  if (!data) 
+  if (size > 0 && !data) 
     {
       if (allow_partial &&
 	  priv->pos < priv->len)
 	return TRUE; /* Short write */
       
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_NO_SPACE,
-                   _("Failed to resize memory output stream"));
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_NO_SPACE,
+                           _("Failed to resize memory output stream"));
       return FALSE;
     }
 
@@ -346,6 +348,9 @@ array_resize (GMemoryOutputStream  *ostream,
   priv->data = data;
   priv->len = size;
   
+  if (priv->len < priv->valid_len)
+    priv->valid_len = priv->len;
+  
   return TRUE;
 }
 
@@ -396,6 +401,9 @@ g_memory_output_stream_write (GOutputStream  *stream,
   dest = (guint8 *)priv->data + priv->pos;
   memcpy (dest, buffer, count); 
   priv->pos += count;
+  
+  if (priv->pos > priv->valid_len)
+    priv->valid_len = priv->pos;
 
   return count;
 }
@@ -539,20 +547,20 @@ g_memory_output_stream_seek (GSeekable    *seekable,
       break;
   
     default:
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Invalid GSeekType supplied"));
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_INVALID_ARGUMENT,
+                           _("Invalid GSeekType supplied"));
 
       return FALSE;
     }
 
   if (absolute < 0) 
     {
-      g_set_error (error,
-                   G_IO_ERROR,
-                   G_IO_ERROR_INVALID_ARGUMENT,
-                   _("Invalid seek request"));
+      g_set_error_literal (error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_INVALID_ARGUMENT,
+                           _("Invalid seek request"));
       return FALSE;
     }
 
@@ -582,11 +590,7 @@ g_memory_output_stream_truncate (GSeekable     *seekable,
                                  GCancellable  *cancellable,
                                  GError       **error)
 {
-  GMemoryOutputStream *ostream;
-  GMemoryOutputStreamPrivate *priv;
-
-  ostream = G_MEMORY_OUTPUT_STREAM (seekable);
-  priv = ostream->priv;
+  GMemoryOutputStream *ostream = G_MEMORY_OUTPUT_STREAM (seekable);
  
   if (!array_resize (ostream, offset, FALSE, error))
     return FALSE;
