@@ -47,6 +47,7 @@
 #include <glib/gfileutils.h>
 #include <glib/ghash.h>
 #include <glib/ghook.h>
+#include <glib/ghostutils.h>
 #include <glib/giochannel.h>
 #include <glib/gkeyfile.h>
 #include <glib/glist.h>
@@ -59,6 +60,7 @@
 #include <glib/gnode.h>
 #include <glib/goption.h>
 #include <glib/gpattern.h>
+#include <glib/gpoll.h>
 #include <glib/gprimes.h>
 #include <glib/gqsort.h>
 #include <glib/gquark.h>
