@@ -204,7 +204,7 @@ g_once_init_enter_impl (volatile gsize *value_location)
 {
   gboolean need_init = FALSE;
   g_mutex_lock (g_once_mutex);
-  if (g_atomic_pointer_get ((void**) value_location) == NULL)
+  if (g_atomic_pointer_get (value_location) == NULL)
     {
       if (!g_slist_find (g_once_init_list, (void*) value_location))
         {
@@ -224,11 +224,11 @@ void
 g_once_init_leave (volatile gsize *value_location,
                    gsize           initialization_value)
 {
-  g_return_if_fail (g_atomic_pointer_get ((void**) value_location) == NULL);
+  g_return_if_fail (g_atomic_pointer_get (value_location) == NULL);
   g_return_if_fail (initialization_value != 0);
   g_return_if_fail (g_once_init_list != NULL);
 
-  g_atomic_pointer_set ((void**) value_location, (void*) initialization_value);
+  g_atomic_pointer_set ((void**)value_location, (void*) initialization_value);
   g_mutex_lock (g_once_mutex);
   g_once_init_list = g_slist_remove (g_once_init_list, (void*) value_location);
   g_cond_broadcast (g_once_cond);
@@ -256,7 +256,7 @@ g_static_mutex_get_mutex_impl (GMutex** mutex)
   g_mutex_lock (g_once_mutex);
 
   if (!(*mutex))
-    g_atomic_pointer_set ((void**) mutex, g_mutex_new());
+    g_atomic_pointer_set (mutex, g_mutex_new());
 
   g_mutex_unlock (g_once_mutex);
 
@@ -979,6 +979,8 @@ g_thread_foreach (GFunc    thread_func,
  *
  * Indicates if g_thread_init() has been called.
  *
+ * Returns: %TRUE if threads have been initialized.
+ *
  * Since: 2.20
  */
 gboolean
